/**
 * Created by perkin on 2017/2/9.
 */
import React , { Component, PropTypes } from 'react'
//video插件
import plyr from 'plyr';
import './VideoContainer.scss'
import '../../styles/plyr.scss'
//
export class VideoContainer extends Component{
    constructor(props){
        super(props)
        this.instance = null;
        this.state = {
        }
    }
    static defaultProps = {
        disabled: false,
        controls: ['play-large', 'play', 'progress', 'current-time', 'mute', 'volume', 'captions', 'fullscreen'],
        loadSprite: true,
        //iconUrl: null,
        iconPrefix: 'plyr',
        debug: false,
        autoplay: false,
        seekTime: 10,
        volume: 5,
        clickToPlay: true,
        disableContextMenu: true,
        hideControls: true,
        showPosterOnEnd: false,
        keyboardShortcuts: {
          focused: true,
          global: false
        },
        tooltips: {
          controls: false,
          seek: true
        },
        duration: null,
        displayDuration: true,
        storage: {
          enabled: true,
          key: 'xkVideoPlyr'
        }
    }
    static propTypes = {
        onReady: PropTypes.func,
        onPlay: PropTypes.func,
        onPause: PropTypes.func,
        onEnd: PropTypes.func,
        onSeeked: PropTypes.func,
        onEnterFullscreen: PropTypes.func,
        onVolumeChange: PropTypes.func,

        // plyr props
        disabled: PropTypes.bool,
        controls: PropTypes.arrayOf(PropTypes.string),
        loadSprite: PropTypes.bool,
        //iconUrl: PropTypes.string,
        iconPrefix: PropTypes.string,
        debug: PropTypes.bool,
        autoplay: PropTypes.bool,
        seekTime: PropTypes.number,
        volume: PropTypes.number,
        clickToPlay: PropTypes.bool,
        disableContextMenu: PropTypes.bool,
        hideControls: PropTypes.bool,
        showPosterOnEnd: PropTypes.bool,
        keyboardShortcuts: PropTypes.shape({
          focused: PropTypes.bool,
          global: PropTypes.bool
        }),
        tooltips: PropTypes.shape({
          controls: PropTypes.bool,
          seek: PropTypes.bool
        }),
        duration: PropTypes.number,
        displayDuration: PropTypes.bool,
        storage: PropTypes.shape({
          enabled: PropTypes.bool,
          key: PropTypes.string
        }),
    }

    componentDidMount(){
        const options = {
              enabled: !this.props.disabled,
              controls: this.props.controls,
              loadSprite: this.props.loadSprite,
              // iconUrl: this.props.iconUrl,
              iconPrefix: this.props.iconPrefix,
              debug: this.props.debug,
              autoplay: this.props.autoplay,
              seekTime: this.props.seekTime,
              volume: this.props.volume,
              clickToPlay: this.props.clickToPlay,
              disableContextMenu: this.props.disableContextMenu,
              hideControls: this.props.hideControls,
              showPosterOnEnd: this.props.showPosterOnEnd,
              keyboardShortcuts: this.props.keyboardShortcuts,
              tooltips: this.props.tooltips,
              duration: this.props.duration,
              displayDuration: this.props.displayDuration,
              storage: this.props.storage
            };

            this.instance = plyr.setup('.xkVideoPlyr', options);

            this.instance[0].on('ready', () => {
              this.props.onReady && this.props.onReady();
            });

            this.instance[0].on('play', () => {
              this.props.onPlay && this.props.onPlay();
            });

            this.instance[0].on('pause', () => {
              this.props.onPause && this.props.onPause();
            });

            this.instance[0].on('ended', () => {
              this.props.onEnd && this.props.onEnd();
            });

            this.instance[0].on('seeked', event => {
              const time = event.detail.plyr.getCurrentTime();
              this.props.onSeeked && this.props.onSeeked(time);
            });

            this.instance[0].on('enterfullscreen', () => {
              this.props.onEnterFullscreen && this.props.onEnterFullscreen();
            });

            this.instance[0].on('exitfullscreen', () => {
              this.props.onExitFullscreen && this.props.onExitFullscreen();
            });

            this.instance[0].on('volumechange', event => {
              const isMuted = event.detail.plyr.isMuted();
              const volume = event.detail.plyr.getVolume();

            this.props.onVolumeChange && this.props.onVolumeChange({ isMuted, volume });
        });
    }
    
    //单个视频播放切换更新props
    componentWillReceiveProps(nextProps){
         if(nextProps.isExpandVideo){
             if(this.props.autoPlay){
                 this.refs.videoContainer.plyr.play()
             }
         }else{
            this.refs.videoContainer.plyr.pause()
         }
    }
    componentWillUnmount() {
        this.instance[0].destroy();
    }

    render(){
        const {autoPlay,fullWindow,onClickSide,onCloseVideo,isExpandVideo,videoUrl,videoInfo,title} = this.props
        return(
            <div>
              <div className="VideoContainer" style={isExpandVideo?{display:'block'}:{display:'none'}}>
                  <div className="top-title">
                      {title}
                      <i 
                        className="close-icon" 
                        onClick={onCloseVideo}>
                      </i>
                  </div>
                  <div style={fullWindow?{left: '0px',top: '55px',height: '457px',width: '900px'}:{}} className="VideoContainer-container">
                      <video
                         className="xkVideoPlyr"
                         src={videoUrl}
                         ref="videoContainer"
                         controls
                         autoPlay={autoPlay}
                         style={fullWindow?
                          {height: '457px',width: '900px'}:
                          {height:'240px'}}
                      >
                      </video>                                         
                  </div>
                  {fullWindow?"":<div className="preview">
                    <p>截图预览</p>
                    <img src={videoInfo ? videoInfo.coverUrl : ''} alt="" />
                  </div>}
              </div>
              {fullWindow?<div 
                style={isExpandVideo?{display:'block'}:{display:'none'}} 
                className="videoContainerBG"
                onClick={onCloseVideo}></div>:""}
            </div>
        )
    }
}
export default VideoContainer